package cn.org.bjca.trust.android.lib.im.request;

import java.lang.reflect.Type;
import java.util.List;

import cn.org.bjca.trust.android.lib.im.common.json.GsonImplHelp;
import cn.org.bjca.trust.android.lib.im.common.json.reflect.TypeBuilder;
import cn.org.bjca.trust.android.lib.im.http.HttpResult;
import cn.org.bjca.trust.android.lib.im.im.IMHelper;
import cn.org.bjca.trust.android.lib.im.im.kit.RequestImCallback;
import cn.org.bjca.trust.android.lib.im.im.msg.RequestMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.ResultMessage;

public class RequestManager implements RequestInterface {

    @Override
    public <T> void request(String path, Object data, Class<T> clazz, RequestCallback<T> callback) {
        performRequest(path, data, clazz, callback);
    }

    @Override
    public <T> void requestList(String path, Object data, Class<T> clazz, RequestCallback<List<T>> callback) {
        performRequest(path, data, TypeBuilder.newInstance(List.class).addTypeParam(clazz).build(), callback);
    }

    private <T> void performRequest(String path, Object data, Type type, RequestCallback<T> callback) {
        if (callback == null) {
            return;
        }

        RequestMessage rm = new RequestMessage(path, GsonImplHelp.get().toJson(data));

        IMHelper.getInstance().request(rm, new RequestImCallback() {

            @Override
            public void success(ResultMessage message) {
                try {
                    if (message == null || message.getBody() == null) {
                        callback.failed(-1, "Empty response");
                        return;
                    }
                    HttpResult<T> h = GsonImplHelp.get().toObject(message.getBody(), TypeBuilder.newInstance(HttpResult.class).addTypeParam(type).build());
                    if (h == null || h.getCode() != 200) {
                        callback.failed(h != null ? h.getCode() : -1, h != null ? h.getMsg() : "Unknown error");
                    } else {
                        callback.success(h);
                    }
                } catch (Exception e) {
                    callback.failed(-1, "Error parsing response: " + e.getMessage());
                }
            }

            @Override
            public void failed(int code, String error) {
                if (code == 400504) {
                    callback.standby(new RequestCallback<List<T>>() {
                        @Override
                        public void success(HttpResult<List<T>> result) {
                            callback.success(result);
                        }

                        @Override
                        public void failed(int code, String error) {
                            callback.failed(code, error);
                        }

                        @Override
                        public void standby(RequestCallback<List<T>> imCallback) {

                        }
                    });
                } else {
                    callback.failed(code, error);
                }
            }
        });
    }

}
